package fr.mael.jiwigo.transverse.util;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;

import com.drew.imaging.jpeg.JpegMetadataReader;
import com.drew.imaging.jpeg.JpegProcessingException;
import com.drew.metadata.Directory;
import com.drew.metadata.Metadata;
import com.drew.metadata.MetadataException;
import com.drew.metadata.Tag;

public class MetadataUtil {
    public static HashMap<Integer, String> getIptcMap() {
	HashMap<Integer, String> tagNameMap = new HashMap<Integer, String>();
	//	tagNameMap.put(new Integer(512), "Directory Version");
	tagNameMap.put(new Integer(632), "iptc_Caption_Abstract");
	tagNameMap.put(new Integer(634), "iptc_Writer_Editor");
	tagNameMap.put(new Integer(617), "iptc_Headline");
	tagNameMap.put(new Integer(552), "iptc_Special_Instructions");
	tagNameMap.put(new Integer(592), "iptc_By_line");
	tagNameMap.put(new Integer(597), "iptc_By_line_Title");
	tagNameMap.put(new Integer(622), "iptc_Credit");
	tagNameMap.put(new Integer(627), "iptc_Source");
	tagNameMap.put(new Integer(517), "iptc_Object");
	tagNameMap.put(new Integer(567), "iptc_Date_Created");
	tagNameMap.put(new Integer(602), "iptc_City");
	tagNameMap.put(new Integer(607), "iptc_Province_State");
	tagNameMap.put(new Integer(613), "iptc_Country_Primary_Location_Code");
	tagNameMap.put(new Integer(615), "iptc_Original_Transmission_Reference");
	tagNameMap.put(new Integer(527), "iptc_Category");
	tagNameMap.put(new Integer(532), "iptc_Supplemental_Category");
	tagNameMap.put(new Integer(522), "iptc_Urgency");
	tagNameMap.put(new Integer(537), "iptc_Keywords");
	tagNameMap.put(new Integer(628), "iptc_Copyright_Notice");
	tagNameMap.put(new Integer(542), "iptc_Release_Date");
	tagNameMap.put(new Integer(547), "iptc_Release_Time");
	tagNameMap.put(new Integer(572), "iptc_Time_Created");
	tagNameMap.put(new Integer(577), "iptc_Originating_Program");
	return tagNameMap;
    }

    public static HashMap<Integer, String> getExifMap() {
	HashMap<Integer, String> tagNameMap = new HashMap<Integer, String>();
	tagNameMap.put(new Integer(266), "exif_field_FillOrder");
	tagNameMap.put(new Integer(269), "exif_field_DocumentName");
	tagNameMap.put(new Integer(4096), "exif_field_RelatedImageFileFormat");
	tagNameMap.put(new Integer(4097), "exif_field_RelatedImageWidth");
	tagNameMap.put(new Integer(4098), "exif_field_RelatedImageLength");
	tagNameMap.put(new Integer(342), "exif_field_TransferRange");
	tagNameMap.put(new Integer(512), "exif_field_JPEGProc");
	tagNameMap.put(new Integer(34665), "exif_field_ExifOffset");
	tagNameMap.put(new Integer(37122), "exif_field_CompressedBitsPerPixel");
	//	tagNameMap.put(new Integer(37500), "Maker Note");
	tagNameMap.put(new Integer(40965), "exif_field_InteropOffset");
	//	tagNameMap.put(new Integer(254), "New Subfile Type");
	tagNameMap.put(new Integer(255), "exif_field_SubfileType");
	//	tagNameMap.put(new Integer(256), "Thumbnail Image Width");
	//	tagNameMap.put(new Integer(257), "Thumbnail Image Height");
	tagNameMap.put(new Integer(258), "exif_field_BitsPerSample");
	tagNameMap.put(new Integer(259), "exif_field_Compression");
	tagNameMap.put(new Integer(262), "exif_field_PhotometricInterpretation");
	tagNameMap.put(new Integer(263), "exif_field_Thresholding");
	tagNameMap.put(new Integer(270), "exif_field_ImageDescription");
	tagNameMap.put(new Integer(271), "exif_field_Make");
	tagNameMap.put(new Integer(272), "exif_field_Model");
	tagNameMap.put(new Integer(273), "exif_field_StripOffsets");
	tagNameMap.put(new Integer(274), "exif_field_Orientation");
	tagNameMap.put(new Integer(277), "exif_field_SamplesPerPixel");
	tagNameMap.put(new Integer(278), "exif_field_RowsPerStrip");
	tagNameMap.put(new Integer(279), "exif_field_StripByteCounts");
	tagNameMap.put(new Integer(282), "exif_field_XResolution");
	tagNameMap.put(new Integer(283), "exif_field_YResolution");
	tagNameMap.put(new Integer(285), "exif_field_PageName");
	tagNameMap.put(new Integer(284), "exif_field_PlanarConfiguration");
	tagNameMap.put(new Integer(296), "exif_field_ResolutionUnit");
	tagNameMap.put(new Integer(301), "exif_field_TransferFunction");
	tagNameMap.put(new Integer(305), "exif_field_Software");
	tagNameMap.put(new Integer(306), "exif_field_TakenDate");
	tagNameMap.put(new Integer(315), "exif_field_Artist");
	tagNameMap.put(new Integer(317), "exif_field_Predictor");
	tagNameMap.put(new Integer(318), "exif_field_WhitePoint");
	tagNameMap.put(new Integer(319), "exif_field_PrimaryChromaticities");
	tagNameMap.put(new Integer(322), "exif_field_TileWidth");
	tagNameMap.put(new Integer(323), "exif_field_TileLength");
	tagNameMap.put(new Integer(324), "exif_field_TileOffsets");
	tagNameMap.put(new Integer(325), "exif_field_TileByteCounts");
	tagNameMap.put(new Integer(330), "exif_field_SubIFD");
	tagNameMap.put(new Integer(347), "exif_field_JPEGTables");
	tagNameMap.put(new Integer(513), "exif_field_ThumbnailOffset");
	tagNameMap.put(new Integer(514), "exif_field_ThumbnailLength");
	//	tagNameMap.put(new Integer(61441), "Thumbnail Data");
	tagNameMap.put(new Integer(529), "exif_field_YCbCrCoefficients");
	tagNameMap.put(new Integer(530), "exif_field_YCbCrSubSampling");
	tagNameMap.put(new Integer(531), "exif_field_YCbCrPositioning");
	tagNameMap.put(new Integer(532), "exif_field_ReferenceBlackWhite");
	tagNameMap.put(new Integer(33421), "exif_field_CFARepeatPatternDim");
	tagNameMap.put(new Integer(33422), "exif_field_CFAPattern2");
	tagNameMap.put(new Integer(33423), "exif_field_BatteryLevel");
	tagNameMap.put(new Integer(33432), "exif_field_Copyright");
	tagNameMap.put(new Integer(33434), "exif_field_ExposureTime");
	tagNameMap.put(new Integer(33437), "exif_field_FNumber");
	tagNameMap.put(new Integer(33723), "exif_field_IPTC-NAA");
	tagNameMap.put(new Integer(34675), "exif_field_ICC_Profile");
	tagNameMap.put(new Integer(34850), "exif_field_ExposureProgram");
	tagNameMap.put(new Integer(34852), "exif_field_SpectralSensitivity");
	tagNameMap.put(new Integer(34853), "exif_field_GPSInfo");
	tagNameMap.put(new Integer(34855), "exif_field_ISOSpeedRatings");
	tagNameMap.put(new Integer(34856), "exif_field_Opto-ElectricConvFactor");
	tagNameMap.put(new Integer(34857), "exif_field_Interlace");
	tagNameMap.put(new Integer(34858), "exif_field_TimeZoneOffset");
	tagNameMap.put(new Integer(34859), "exif_field_SelfTimer");
	tagNameMap.put(new Integer(36864), "exif_field_ExifVersion");
	tagNameMap.put(new Integer(36867), "exif_field_DateTimeOriginal");
	//	tagNameMap.put(new Integer(36868), "Date/Time Digitized");
	tagNameMap.put(new Integer(37121), "exif_field_ComponentsConfiguration");
	tagNameMap.put(new Integer(37377), "exif_field_ShutterSpeedValue");
	tagNameMap.put(new Integer(37378), "exif_field_ApertureValue");
	tagNameMap.put(new Integer(37379), "exif_field_BrightnessValue");
	tagNameMap.put(new Integer(37380), "exif_field_ExposureBiasValue");
	tagNameMap.put(new Integer(37381), "exif_field_MaxApertureValue");
	tagNameMap.put(new Integer(37382), "exif_field_SubjectDistance");
	tagNameMap.put(new Integer(37383), "exif_field_MeteringMode");
	tagNameMap.put(new Integer(37384), "exif_field_LightSource");
	tagNameMap.put(new Integer(37385), "exif_field_Flash");
	tagNameMap.put(new Integer(37386), "exif_field_FocalLength");
	tagNameMap.put(new Integer(37387), "exif_field_FlashActivity");
	tagNameMap.put(new Integer(37388), "exif_field_SpatialFrequencyResponse");
	tagNameMap.put(new Integer(37389), "exif_field_Noise");
	tagNameMap.put(new Integer(37393), "exif_field_ImageNumber");
	tagNameMap.put(new Integer(37394), "exif_field_SecurityClassification");
	tagNameMap.put(new Integer(37395), "exif_field_ImageHistory");
	tagNameMap.put(new Integer(37396), "exif_field_SubjectLocation");
	tagNameMap.put(new Integer(41493), "exif_field_ExposureIndex");
	tagNameMap.put(new Integer(37398), "exif_field_TIFF-EPStandardID");
	tagNameMap.put(new Integer(37510), "exif_field_UserComment");
	tagNameMap.put(new Integer(37520), "exif_field_SubSecTime");
	tagNameMap.put(new Integer(37521), "exif_field_SubSecTimeOriginal");
	tagNameMap.put(new Integer(37522), "exif_field_SubSecTimeDigitized");
	tagNameMap.put(new Integer(40960), "exif_field_FlashpixVersion");
	tagNameMap.put(new Integer(40961), "exif_field_ColorSpace");
	tagNameMap.put(new Integer(40962), "exif_field_ExifImageWidth");
	//	tagNameMap.put(new Integer(40963), "Exif Image Height");
	tagNameMap.put(new Integer(40964), "exif_field_RelatedSoundFile");
	tagNameMap.put(new Integer(41483), "exif_field_FlashEnergy");
	tagNameMap.put(new Integer(41484), "exif_field_SpatialFrequencyResponse");
	tagNameMap.put(new Integer(41486), "exif_field_FocalPlaneXResolution");
	tagNameMap.put(new Integer(41487), "exif_field_FocalPlaneYResolution");
	tagNameMap.put(new Integer(41488), "exif_field_FocalPlaneResolutionUnit");
	tagNameMap.put(new Integer(41492), "exif_field_SubjectLocation");
	tagNameMap.put(new Integer(37397), "exif_field_ExposureIndex");
	tagNameMap.put(new Integer(41495), "exif_field_SensingMethod");
	tagNameMap.put(new Integer(41728), "exif_field_FileSource");
	tagNameMap.put(new Integer(41729), "exif_field_SceneType");
	tagNameMap.put(new Integer(41730), "exif_field_CFAPattern");
	tagNameMap.put(new Integer(41985), "exif_field_CustomRendered");
	tagNameMap.put(new Integer(41986), "exif_field_ExposureMode");
	tagNameMap.put(new Integer(41987), "exif_field_WhiteBalance");
	tagNameMap.put(new Integer(41988), "exif_field_DigitalZoomRatio");
	tagNameMap.put(new Integer(41989), "exif_field_FocalLengthIn35mmFormat");
	tagNameMap.put(new Integer(41990), "exif_field_SceneCaptureType");
	tagNameMap.put(new Integer(41991), "exif_field_GainControl");
	tagNameMap.put(new Integer(41992), "exif_field_Contrast");
	tagNameMap.put(new Integer(41993), "exif_field_Saturation");
	tagNameMap.put(new Integer(41994), "exif_field_Sharpness");
	tagNameMap.put(new Integer(41995), "exif_field_DeviceSettingDescription");
	tagNameMap.put(new Integer(41996), "exif_field_SubjectDistanceRange");
	//	tagNameMap.put(new Integer(40093), "Windows XP Author");
	//	tagNameMap.put(new Integer(40092), "Windows XP Comment");
	//	tagNameMap.put(new Integer(40094), "Windows XP Keywords");
	//	tagNameMap.put(new Integer(40095), "Windows XP Subject");
	//	tagNameMap.put(new Integer(40091), "Windows XP Title");
	tagNameMap.put(new Integer(280), "exif_field_MinSampleValue");
	tagNameMap.put(new Integer(281), "exif_field_MaxSampleValue");
	return tagNameMap;
    }

    public static HashMap<String, String> getTagsHashMap(String fichier) throws JpegProcessingException,
	    MetadataException {
	File jpegFile = new File(fichier);
	Metadata metadata = JpegMetadataReader.readMetadata(jpegFile);
	Iterator directories = metadata.getDirectoryIterator();
	HashMap<String, String> mapTags = new HashMap<String, String>();
	while (directories.hasNext()) {
	    Directory directory = (Directory) directories.next();
	    Iterator tags = directory.getTagIterator();
	    if (directory.getName().equals("Exif") || directory.getName().equals("Iptc")) {
		while (tags.hasNext()) {
		    Tag tag = (Tag) tags.next();
		    if (tag.getDirectoryName().equals("Exif")) {
			for (Integer key : MetadataUtil.getExifMap().keySet()) {
			    if (tag.getTagType() == key) {
				mapTags.put(MetadataUtil.getExifMap().get(key), tag.getDescription());
			    }
			}
		    } else {
			for (Integer key : MetadataUtil.getIptcMap().keySet()) {
			    if (tag.getTagType() == key) {
				mapTags.put(MetadataUtil.getIptcMap().get(key), tag.getDescription());
			    }
			}
		    }
		}
	    }
	}
	return mapTags;
    }

    public static void main(String[] args) {
	try {
	    HashMap<String, String> map = getTagsHashMap("/home/mael/Images/exif.JPG");
	    for (String key : getTagsHashMap("/home/mael/Images/exif.JPG").keySet()) {
		System.out.println(key + " -> " + map.get(key));
	    }
	} catch (JpegProcessingException e) {
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	} catch (MetadataException e) {
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	}
    }
}
