package fr.mael.jiwigo.transverse.util.preferences;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.jdom.Document;
import org.jdom.input.SAXBuilder;

import fr.mael.jiwigo.transverse.util.Outil;

/**
   Copyright (c) 2010, Mael
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of jiwigo nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL Mael BE LIABLE FOR ANY
   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   
 * Classe de gestion des preferences
 * @author mael
 *
 */
public class PreferencesManagement {

    /**
     * Logger
     */
    public static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory
	    .getLog(PreferencesManagement.class);

    /**
     * fonction qui retourne le chemin du fichier de configuration
     * @return
     */
    private static String getConfigFilePath() {
	return System.getProperty("user.home") + "/.jiwigo/config.xml";
    }

    /**
     * Creation du fichier de configuration par défaut
     * @throws IOException
     */
    public static void creerFichierParDefaut() throws IOException {
	File outFile = new File(getConfigFilePath());
	if (!outFile.exists()) {
	    StringBuffer xml = new StringBuffer();
	    xml.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	    xml.append("<config>\n");
	    xml.append("	<chunk_size>0.5</chunk_size>\n");
	    xml.append("	<width_original>800</width_original>\n");
	    xml.append("	<height_original>600</height_original>\n");
	    xml.append("	<password></password>\n");
	    xml.append("	<login></login>\n");
	    xml.append("	<url></url>\n");
	    xml.append("</config>");

	    FileWriter out = new FileWriter(outFile);
	    out.write(xml.toString());
	    out.close();
	}
    }

    /**
     * Fonction qui retourne la valeur d'une préférence
     * @param key la cle de la preference
     * @return la valeur
     */
    public static String getValue(String key) {
	SAXBuilder sxb = new SAXBuilder();
	try {
	    Document document = sxb.build(new File(getConfigFilePath()));
	    return document.getRootElement().getChildText(key);
	} catch (Exception e) {
	    LOG.error(Outil.getStackTrace(e));
	    return null;
	}
    }

    /**
     * Fonction qui attribue une valeur à une préférence
     * @param key la cle
     * @param text la valeur
     */
    public static void setValue(String key, String text) {
	SAXBuilder sxb = new SAXBuilder();
	try {
	    Document document = sxb.build(new File(getConfigFilePath()));
	    document.getRootElement().getChild(key).setText(text);
	    FileWriter out = new FileWriter(getConfigFilePath());
	    out.write(Outil.documentToString(document));
	    out.close();
	} catch (Exception e) {
	    LOG.error(Outil.getStackTrace(e));
	}
    }
}
