package fr.mael.jiwigo.ui.mainframe;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import fr.mael.jiwigo.transverse.util.Messages;
import fr.mael.jiwigo.transverse.util.Outil;

/**
   Copyright (c) 2010, Mael
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of jiwigo nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL Mael BE LIABLE FOR ANY
   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   
 * @author mael
 * Frame principale
 */
public class MainFrame extends JFrame implements ActionListener {
    /**
     * Logger
     */
    public static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory
	    .getLog(MainFrame.class);
    /**
     * l'arbre des catégories
     */
    private CategoriesTree categoriesTree;
    /**
     * Splitpane avec à gauche l'arbre et à droite les miniatures pour la catégorie
     * courante
     */
    private JSplitPane splitPane;
    /**
     * Panel contenant les miniatures de la catégorie courante
     */
    public static ThumbnailCategoryPanel imagesPanel;
    /**
     *  Scrollpane contenant le panel ci dessus
     */
    private JScrollPane scrollPaneImagesPanel;
    /**
     * label qui affiche des messages
     */
    private JLabel labelMessage;

    /**
     * Barre de menu
     */
    private JMenuBar jMenuBar;

    /**
     * Menu d'édition
     */
    private JMenu jMenuEdition;

    /**
     * menu des preferences
     */
    private JMenuItem jMenuItemPreferences;

    /**
     * singleton
     */
    private static MainFrame instance;

    private JProgressBar progressBar;

    /**
     * @return le singleton
     */
    public static MainFrame getInstance() {
	if (instance == null) {
	    instance = new MainFrame();
	}
	return instance;
    }

    /**
     * COnstructeur privé, permettant de n'avoir accès qu'au singleton
     */
    private MainFrame() {
	this.setTitle("Piwigo v" + Messages.getMessage("version"));
	this.setIconImage(java.awt.Toolkit.getDefaultToolkit().getImage(Outil.getURL("fr/mael/jiwigo/img/icon.png")));
	this.setLayout(new BorderLayout());
	jMenuBar = new JMenuBar();
	labelMessage = new JLabel(Messages.getMessage("welcomeMessage"));
	splitPane = new JSplitPane();
	categoriesTree = new CategoriesTree();
	splitPane.setLeftComponent(categoriesTree);
	imagesPanel = new ThumbnailCategoryPanel(null);
	reduceSizeOfComponent(imagesPanel);
	scrollPaneImagesPanel = new JScrollPane(imagesPanel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	scrollPaneImagesPanel.setPreferredSize(new Dimension(900, 700));
	splitPane.setRightComponent(scrollPaneImagesPanel);

	this.add(splitPane, BorderLayout.WEST);
	JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
	progressBar = new JProgressBar(0, 100);
	panel.add(progressBar);
	panel.add(labelMessage);

	jMenuEdition = new JMenu(Messages.getMessage("mainFrame_editionMenu"));
	jMenuBar.add(jMenuEdition);
	jMenuItemPreferences = new JMenuItem(Messages.getMessage("mainFrame_preferencesMenu"));
	jMenuItemPreferences.addActionListener(this);
	jMenuEdition.add(jMenuItemPreferences);

	this.setJMenuBar(jMenuBar);
	this.add(panel, BorderLayout.SOUTH);
	this.pack();
	this.setLocationRelativeTo(null);
	this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	this.setResizable(false);
    }

    /**
     * affichage de la frame
     */
    public void showUi() {
	this.setVisible(true);
    }

    /**
     * Affichage d'un message d'erreur
     * @param message
     */
    public void setErrorMessage(String message) {
	this.labelMessage.setText(message);
    }

    /**
     * Affichage d'un message de réussite
     * @param message
     */
    public void setReussiteMessage(String message) {
	this.labelMessage.setText(message);
    }

    @Override
    public void actionPerformed(ActionEvent arg0) {
	if (arg0.getSource().equals(jMenuItemPreferences)) {
	    new PreferencesDialog(this);
	}

    }

    public static void reduceSizeOfComponent(JComponent comp) {
	comp.setMaximumSize(comp.getPreferredSize());
    }

    /**
     * @return the progressBar
     */
    public JProgressBar getProgressBar() {
	return progressBar;
    }

}
