package fr.mael.jiwigo.ui.mainframe;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JLabel;

import fr.mael.jiwigo.om.Image;
import fr.mael.jiwigo.transverse.util.Messages;
import fr.mael.jiwigo.ui.ImagesManagement;
import fr.mael.jiwigo.ui.browser.BrowserFrame;

/**
   Copyright (c) 2010, Mael
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of jiwigo nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL Mael BE LIABLE FOR ANY
   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   
 * @author mael
 * Panel contenant la miniature d'une image.
 * C'est un panel "cliquable", qui permet d'afficher l'image à taille réelle
 */
public class ThumbnailPanel extends JLabel implements MouseListener {
    /**
     * Logger
     */
    public static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory
	    .getLog(ThumbnailPanel.class);
    /**
     * l'image
     */
    private Image image;

    /**
     * Constructeur
     * @param image l'image
     * @param imagesPanel le panel
     */
    public ThumbnailPanel(Image image) {
	this.image = image;
	setToolTipText("<html><center>" + image.getName() + "<br/>" + image.getVue() + " "
		+ Messages.getMessage("hits") + "</center></html>");
	this.addMouseListener(this);
    }

    protected void paintComponent(Graphics g) {
	super.paintComponent(g);
	Graphics2D g2 = (Graphics2D) g;
	g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
	double x = (getWidth() * ImagesManagement.getMiniatureBufferedImage(image).getWidth()) / 2;
	double y = (getHeight() * ImagesManagement.getMiniatureBufferedImage(image).getHeight()) / 2;
	g2.drawRenderedImage(ImagesManagement.getMiniatureBufferedImage(image), null);
    }

    /* (non-Javadoc)
     * @see javax.swing.JComponent#getPreferredSize()
     */
    public Dimension getPreferredSize() {
	int w = (int) (ImagesManagement.getMiniatureBufferedImage(image).getWidth());
	int h = (int) (ImagesManagement.getMiniatureBufferedImage(image).getHeight() + 10);
	return new Dimension(w, h);
    }

    @Override
    public void mouseClicked(MouseEvent paramMouseEvent) {
	// on affiche l'image en grand
	ImagesManagement.setCurrentImage(image);
	try {
	    new BrowserFrame(image);
	} catch (Exception e) {

	}

    }

    @Override
    public void mouseEntered(MouseEvent paramMouseEvent) {
	//changement du curseur, pour signifier à l'utilisateur que l'image est cliquable
	this.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    }

    @Override
    public void mouseExited(MouseEvent paramMouseEvent) {
    }

    @Override
    public void mousePressed(MouseEvent paramMouseEvent) {
	// TODO Auto-generated method stub

    }

    @Override
    public void mouseReleased(MouseEvent paramMouseEvent) {
	// TODO Auto-generated method stub

    }

}
